/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/

#ifndef _GALIL1800_EVENT_H_
#define _GALIL1800_EVENT_H_

#ifndef __KERNEL__
#error UserMode programs should include galil1800.h rather than \
							galil1800_event.h
#endif

#include "galil1800.h"

/* Size of one buffer frame:
    to set this value you must know that:
        - It must be a power of 2
        - In reading, the device function read a frame at time
        - Each time the buffer is full and needs to overwrite,
          a whole frame is lost
    So this value can't be too little or too big!!  */
#define GALIL1800_EVENT_BUFF_FRAME_SZ 64

/* the default size of the driver's Kernel Mode buffer
   of interrupt status values and other special driver events.
   This macro is calculated for a default size of whole buffer
   of one page frame (4096 bytes):
   - each event require 2 bytes, so:
                    4096 / 2 = 2048 events in one page frame
   - buffer is split in frames of size GALIL1800_EVENT_BUFF_FRAME_SZ (now 64),
               so:  2048 / 64 = 32 frames in one page frame
   - one frame is used for special purpose (see later), so
                    32 - 1 = 31 frames available for write in one page frame
   N.B.: Attention!! Do not use less then 2 frames!!
*/
#define GALIL1800_EVENT_BUFF_DEFAULT_SZ (GALIL1800_EVENT_BUFF_FRAME_SZ * 31)

#define GALIL1800_EVENT_BUFF_SZ_MIN (GALIL1800_EVENT_BUFF_FRAME_SZ * 2)

/* 1024 page frame */
#define GALIL1800_EVENT_BUFF_SZ_MAX \
		(GALIL1800_EVENT_BUFF_FRAME_SZ * ((32*1024)-1))

/* Size of buffer table (each entry point to one buffer frame) */
#define GALIL1800_EVENT_BUFF_TABLE_SZ(_buff_p) \
	((_buff_p)->size/GALIL1800_EVENT_BUFF_FRAME_SZ)


/* Buffer for event (irq or some other special event).
   Circular Buffer:
	size : Number of event that can be saved into the buffer
	opt_flag : bitmap that define event type to report
	frames : pointer to buffer start address. All this buffer is spitted in
		GALIL1800_EVENT_BUFF_TABLE_SZ() + 1 frames, each one lenght
		GALIL1800_EVENT_BUFF_FRAME_SZ. One frame in each moment is
		anavalaible to write, but it is the last read and in
		each reading. I swap the next to read whit this free frame,
		so I can read whitout lock the buffer.
        table : pointer to an array of pointer that define a circular buffer.
		The array store GALIL1800_EVENT_BUFF_TABLE_SZ() address of
		buffer frame the must be write in sequence, starting by
		table[first_frame]
	first_frame : index in table array of first frame to read or to write
	read_frame : pointer to one buffer frame that is the current
		     (or last) read
	written : current quantity of entries written
	written == 0 : empty buffer
	written == size : full buffer, but buffer can be overwritten
	overwritten_frames : number of frames overwritten

	While reading, if overwritten_frames > 0, first at all write in user
	buffer a GALIL1800_EVENT_BUFF_OVERWRITTEN event and in data field
	the number of record lost.
	If this number is more than 255 (data field is 8 bit) than write 0 in
	data field.

	lock : protect only write operation and a small phase of read, that is
		when we must swap the actual read_frame whit the next one to
		read.
	N.B.: We can use spin_lock whit "spin_lock_bh" and "spin_unlock_bh",
		so whit interrupts enabled, because the (top half) interrupt
		handler do not access to this buffer, but set a bottom half
		handler to add the status value. Because bottom half handler
		is a tasklet, we need of spin_lock_bh for disable soft IRQ.
*/

struct galil1800_event_buffer {
	unsigned int size; /* events that can be saved without overwrite */
	unsigned char opt_flag; /*bitmap: type of info that device can report*/
	struct galil1800_event_data *frames; /* all frames */
	struct galil1800_event_data **table;/*buffer table for circular buffer*/
	unsigned int first_frame; /* index of a frame in table: first to read */
	struct galil1800_event_data *read_frame; /* buffer frame in reading */
	unsigned int written; /* events reported (present size of buffer) */
	unsigned int overwritten_frames; /* number of frames overwritten */
	spinlock_t lock_pt; /* protect pointer to data */
	wait_queue_head_t read_wait; /* waiting for device is ready to read */
	struct class_device_attribute cd_attr_size; /* to set "size" by sysfs */
};

/* First step of initialization: must be call in probe function */
#define init_galil1800_event_buffer(_buff_p, _size)                          \
do {                                                                         \
	(_buff_p)->size = (_size);                                           \
	(_buff_p)->opt_flag = 0;                                             \
	(_buff_p)->frames = 0;                                               \
	(_buff_p)->table = 0;                                                \
	(_buff_p)->first_frame = 0;                                          \
	(_buff_p)->read_frame = 0;                                           \
	(_buff_p)->written = 0;                                              \
	(_buff_p)->overwritten_frames = 0;                                   \
	spin_lock_init(&(_buff_p)->lock_pt);                                 \
	init_waitqueue_head(&(_buff_p)->read_wait);                          \
}while(0)


#define galil1800_event_buff_empty(_buff_p) ((_buff_p)->written == 0)


/* Active report of an event */
#define galil1800_event_active(_buff_p, _event_type)              \
		set_bit((_event_type), (void*)&(_buff_p)->opt_flag)

/* Disactive report of an event */
#define galil1800_event_disactive(_buff_p, _event_type)           \
		clear_bit((_event_type), (void*)&(_buff_p)->opt_flag)

/* Test if a report of an event is active */
#define galil1800_event_is_active(_buff_p, _event_type)           \
		test_bit((_event_type), (void*)&(_buff_p)->opt_flag)

/* Set to report all event */
#define galil1800_event_active_all(_buff_p)                                  \
do {                                                                         \
	int nr_bits=0;                                                       \
	while(nr_bits < (sizeof((_buff_p)->opt_flag)*8))                     \
		set_bit(nr_bits++, (void*)&(_buff_p)->opt_flag);             \
	GALIL_PRINT_DEBUG("opt_flag: active all %x\n", (_buff_p)->opt_flag); \
}while(0)

/* Set to not report any event */
#define galil1800_event_disactive_all(_buff_p) do {                            \
	int nr_bits=0;                                                         \
	while(nr_bits < (sizeof((_buff_p)->opt_flag)*8))                       \
		clear_bit(nr_bits++, (void*)&(_buff_p)->opt_flag);             \
	GALIL_PRINT_DEBUG("opt_flag: disactive all %x\n", (_buff_p)->opt_flag);\
}while(0)

/* Set default event to report */
#define galil1800_event_active_default(_buff_p) do {                         \
	galil1800_event_active(_buff_p, GALIL1800_EVENT_IRQ);                \
	galil1800_event_active(_buff_p, GALIL1800_EVENT_WBUFF_LESS_10P);     \
	GALIL_PRINT_DEBUG("opt_flag: active default %x\n",                   \
				(_buff_p)->opt_flag);                        \
}while(0)

/* Return only card_wbuff bit of opt_flag */
#define galil1800_event_get_wbuff_flag(_buff_p)              \
	( (_buff_p)->opt_flag & ((unsigned char) (           \
		(1 << GALIL1800_EVENT_WBUFF_LESS_10P) |      \
		(1 << GALIL1800_EVENT_WBUFF_LESS_20P) |      \
		(1 << GALIL1800_EVENT_WBUFF_LESS_50P) |      \
		(1 << GALIL1800_EVENT_WBUFF_OVER_90P) ) ) )


/* If flag is active acquire the spinlock and add info in the buffer
   Return 1 if add, 0 otherwise */
#define __galil1800_event_buff_lock_and_add(_buff_p, _info, _event_type)      \
({                                                                            \
	int __ret;                                                            \
	NO_GALIL_PRINT_DEBUG("Request to add event %d\n", _event_type);       \
	if ((__ret=galil1800_event_is_active(_buff_p, _event_type))) {        \
		spin_lock_bh(&(_buff_p)->lock_pt);                            \
		/* I must retest because event device release */              \
		/* function can free the buffer! */                           \
		if ((__ret=galil1800_event_is_active(_buff_p, _event_type)))  \
			__galil1800_event_buff_add(_buff_p,_info,_event_type);\
		spin_unlock_bh(&(_buff_p)->lock_pt);                          \
		wake_up_interruptible_sync(&(_buff_p)->read_wait);            \
	}                                                                     \
	__ret;                                                                \
})


/* Increment the first_frame index */
#define galil1800_event_buff_first_frame_inc(_buff_p) do {                    \
	(_buff_p)->first_frame =                                              \
	((_buff_p)->first_frame + 1)<GALIL1800_EVENT_BUFF_TABLE_SZ(_buff_p) ? \
					((_buff_p)->first_frame + 1) : 0;     \
}while(0)

/* add info in the buffer (data=info and type=info_type) */
#define __galil1800_event_buff_add(_buff_p, _info, _info_type)             \
do {                                                                       \
	struct galil1800_event_data *next_to_write;                        \
	if ((_buff_p)->written == (_buff_p)->size) {                       \
		/* Full Buffer, so overwrite! */                           \
		galil1800_event_buff_first_frame_inc(_buff_p);             \
		(_buff_p)->written -= GALIL1800_EVENT_BUFF_FRAME_SZ;       \
		(_buff_p)->overwritten_frames++;                           \
	}                                                                  \
	next_to_write = (_buff_p)->table [                                 \
	((_buff_p)->first_frame +                                          \
	((_buff_p)->written/GALIL1800_EVENT_BUFF_FRAME_SZ)) %              \
				GALIL1800_EVENT_BUFF_TABLE_SZ(_buff_p) ]   \
		+ ((_buff_p)->written%GALIL1800_EVENT_BUFF_FRAME_SZ);      \
	NO_GALIL_PRINT_DEBUG("next_to write %lx\n",                        \
			(unsigned long)next_to_write);                     \
	next_to_write->type = (_info_type);                                \
	next_to_write->data = (_info);                                     \
	NO_GALIL_PRINT_DEBUG("Add event %02x %02x\n",                      \
	next_to_write->type, next_to_write->data);                         \
	barrier();                                                         \
	/*For wait queue check, it's important don't reorder this op.*/    \
	(_buff_p)->written++;                                              \
	NO_GALIL_PRINT_DEBUG("(add) written %u\n", (_buff_p)->written);    \
}while(0);


/* Return 1 if add, 0 otherwise */
#define galil1800_event_buff_add_wbuff_less_10p(_buff_p, _info)        \
			__galil1800_event_buff_lock_and_add(_buff_p,   \
				_info, GALIL1800_EVENT_WBUFF_LESS_10P)

/* Return 1 if add, 0 otherwise */
#define galil1800_event_buff_add_wbuff_less_20p(_buff_p, _info)        \
			__galil1800_event_buff_lock_and_add(_buff_p,   \
				_info, GALIL1800_EVENT_WBUFF_LESS_20P)

/* Return 1 if add, 0 otherwise */
#define galil1800_event_buff_add_wbuff_less_50p(_buff_p, _info)        \
			__galil1800_event_buff_lock_and_add(_buff_p,   \
				_info, GALIL1800_EVENT_WBUFF_LESS_50P)

/* Return 1 if add, 0 otherwise */
#define galil1800_event_buff_add_wbuff_over_90p(_buff_p, _info)        \
			__galil1800_event_buff_lock_and_add(_buff_p,   \
				_info, GALIL1800_EVENT_WBUFF_OVER_90P)



/* IRQ Manage: */

/* the size of the driver's Kernel Mode stack of interrupt status bytes */
#define GALIL1800_IRQ_STACK_SZ 2048

/* Top half interrupt handler save here the "interrupt status byte" and
   bottom half interrupt handler add this data to general event buffer */
struct galil1800_irq_stack {
	unsigned char data[GALIL1800_IRQ_STACK_SZ];
	unsigned char *head; /* next to write */
	unsigned char *tail; /* next to read */
};

#define init_galil1800_irq_stack(_pt) do {               \
	memset((_pt)->data, 0, GALIL1800_IRQ_STACK_SZ);  \
	(_pt)->head = (_pt)->tail = (_pt)->data;         \
}while(0)

#define galil1800_irq_stack_add(_pt, _status_irq) do {                  \
	unsigned char *new_head =                                       \
	(_pt)->head + 1 < (_pt)->data + GALIL1800_IRQ_STACK_SZ ?        \
					(_pt)->head + 1 : (_pt)->data;  \
	if(new_head != (_pt)->tail) {                                   \
		*((_pt)->head)=(_status_irq);                           \
		(_pt)->head = new_head;                                 \
	}                                                               \
}while(0)

#define for_each_galil1800_irq_stack(_pt)                                   \
	for( ; (_pt)->head != (_pt)->tail; (_pt)->tail =                    \
		((_pt)->tail + 1 < (_pt)->data + GALIL1800_IRQ_STACK_SZ ?   \
					(_pt)->tail + 1 : (_pt)->data) )    \


/* Top half interrupt handlers */
extern irqreturn_t galil1800_interrupt(int, void *, struct pt_regs *);

/* This is the bottom half interrupt handler */
extern void galil1800_bottom_half_interrupt(unsigned long data);

/* Add event cdev device and register it in sysfs */
extern int galil1800_event_cdev_add(struct cdev *cdev_p,
			unsigned int major, unsigned int minor);

/* Delete event cdev device and deregister it in sysfs */
extern void galil1800_event_cdev_del(struct cdev *cdev_p);

/* Check if value set for event buffer size (*ev_buff_sz)
    is correct and eventually correct it
    Return 0 if no correct is needed, otherwise a positive value */
extern int galil1800_event_buff_size_check(unsigned int *ev_buff_sz);

#endif
