/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/

#include "galil1800_io.h"

/* Read max len bytes from the read fifo */
ssize_t galil1800_recv(struct galil1800_state *s, char *buf, ssize_t len)
{
	int count;

	if (!galil1800_may_read(s))
		return 0;

	count = 0;
	do {
		*buf++ = inb(GALIL1800_READ_REG(s));
		++count;
	} while(count < len && __galil1800_may_read(s));

	return count;
}

/* write max len bytes to the write fifo */
ssize_t galil1800_send(struct galil1800_state *s, const char *buf, size_t len)
{
	size_t count = len;
	int i, burst;
	const char *p = buf;

	burst = galil1800_may_write(s);

	if(!count || !burst)
		return 0;

	if(burst == 2) {
		do {
			burst = count > GALIL1800_HALF_FULL_WRITE_BURST ?
				GALIL1800_HALF_FULL_WRITE_BURST : count;

			for (i=0; i < burst; ++i, ++p)
				outb_p(*p, GALIL1800_WRITE_REG(s));

			count -= burst;

		} while(count && __galil1800_may_write(s) == 2);
	}

	while(count && __galil1800_may_write(s)) {
		outb(*p, GALIL1800_WRITE_REG(s));
		--count;
		++p;
	}

	return len-count;
}

/* If this macro is defined fifo 2 is read 4 bytes at time.
   Comment thi macro to read fifo 2 one byte at time */
#define __GALIL1800_FIFO2_BURST__

/* Read from the secondary communication channel (2nd FIFO)
   - data are written in @buf, a preallocated buffer of size @len
   - return the number of bytes read (even 0) */
ssize_t galil1800_read_FIFO2(struct galil1800_state *s, char *buf, size_t len)
{
	char *pc = buf;
	ssize_t count = len;
#ifdef __GALIL1800_FIFO2_BURST__
	unsigned long *pl = (unsigned long*)buf;
	size_t adjust = len%4;
	count -= adjust;
#endif

	/* if secondary fifo is disabled */
	if (!__galil1800_may_read_FIFO2(s))
		return 0;

	galil1800_freeze_FIFO2(s);

	galil1800_wait_transfer_FIFO2(s);

	do {
	#ifndef __GALIL1800_FIFO2_BURST__
		*pc++ = inb(GALIL1800_2CHAN_REG(s));
		--count;
	#else
		*pl++ = inl(GALIL1800_2CHAN_REG(s));
		count-=4;
	#endif
	} while (count && galil1800_may_read_FIFO2(s));

	#ifdef __GALIL1800_FIFO2_BURST__
	pc = (char*)pl;
	count += adjust;
	while(count && galil1800_may_read_FIFO2(s)) {
		*pc++ = inb(GALIL1800_2CHAN_REG(s));
		--count;
	}
	#endif

	galil1800_unfreeze_FIFO2(s);

	return len-count;
}
