/* Copyright (C) 2005-2006  Emiliano Betti and Marco Cesati
   Copyright (C) 2003-2004  Marco Cesati
   Copyright (C) 2002-2003  Shell Technologies s.r.l.

   Authors:
            Emiliano Betti <emilianobetti@gmail.com>
            Marco Cesati   <marcocesati@gmail.com>

   Driver for the Galil 1800 PCI motion controllers (http://www.galilmc.com/)

   This is a driver for the Linux kernel 2.6. It is based on an older driver
   for the kernel 2.4, which in turn was loosely inspired by Galil's own
   Linux driver, as well as by a bunch of PCI drivers in the kernel source code

   This program is free software; you can redistribute  it and/or modify it
   under  the terms of  the GNU General  Public License as published by the
   Free Software Foundation.
*/


#ifndef _GALIL1800_IO_H_
#define _GALIL1800_IO_H_

#ifndef __KERNEL__
#error UserMode programs should include galil1800.h rather than galil1800_io.h
#endif

#include "galil1800_core.h"

#include <asm/io.h>
#include <linux/sched.h>
#include <linux/delay.h>

#define PLX_PCI_VENDOR_ID            0x10B5
#define PLX_PCI_DEVICE_ID            0x9050
#define GALIL_SUB_VENDOR             0x1079
#define GALIL_SUB_DEVICE_1800        0x1800

/* The Base Address Register index in the PCI configuration space */
#define GALIL1800_DPRAM_BAR_INDEX           0
#define GALIL1800_REG_BAR_INDEX             2

/* The PCI resource lenght at REG BAR */
#define GALIL1800_REG_PCI_RESOURCE_LEN   16


/* default milliseconds for [read,write,fifo2] timeout.
   It is initialized in milliseconds and user must set it in ms,
   after, in module init function or ioctl call, it is checked and converted! */
#define GALIL1800_READ_TIMEOUT_MS_DEFAULT       20
#define GALIL1800_WRITE_TIMEOUT_MS_DEFAULT      20
#define GALIL1800_FIFO2_TIMEOUT_MS_DEFAULT      20

/* Check if value set for write_timeout from user
    is correct and eventually correct it
    After convert it from ms to ticks!!
    Return 0 if ok, -EINVAL if reset to default */
static
inline int galil1800_write_timeout_check_and_convert(unsigned int *timeout)
{
	int ret=0;
	if(*timeout < 1) {
		*timeout = GALIL1800_WRITE_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "write_timeout is less then 1,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	} else if(*timeout > 500) {
		*timeout = GALIL1800_WRITE_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "write_timeout is more then 500,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	}
	*timeout = ((*timeout)*HZ)/1000; /* Convert from ms to ticks */
	return ret;
}

/* Check if value set for read_timeout from user
    is correct and eventually correct it
    After convert it from ms to ticks!!
    Return 0 if ok, -EINVAL if reset to default */
static
inline int galil1800_read_timeout_check_and_convert(unsigned int *timeout)
{
	int ret=0;
	if(*timeout < 1) {
		*timeout = GALIL1800_READ_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "read_timeout is less then 1,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	} else if(*timeout > 500) {
		*timeout = GALIL1800_READ_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "read_timeout is more then 500,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	}
	*timeout = ((*timeout)*HZ)/1000; /* Convert from ms to ticks */
	return ret;
}

/* Check if value set for fifo2_timeout from user
    is correct and eventually correct it
    After convert it from ms to ticks!!
    Return 0 if ok, -EINVAL if reset to default */
static
inline int galil1800_fifo2_timeout_check_and_convert(unsigned int *timeout)
{
	int ret=0;
	if(*timeout < 1) {
		*timeout = GALIL1800_FIFO2_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "fifo2_timeout is less then 1,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	} else if(*timeout > 500) {
		*timeout = GALIL1800_FIFO2_TIMEOUT_MS_DEFAULT;
		printk(KERN_NOTICE PFX "fifo2_timeout is more then 500,"
				" set default value (%d)\n", *timeout);
		ret=-EINVAL;
	}
	*timeout = ((*timeout)*HZ)/1000; /* Convert from ms to ticks */
	return ret;
}


/* default milliseconds to wait for check available space in the write fifo.
   It is initialized in milliseconds and user must set it in ms,
   after, in module init function or ioctl call, it is checked and converted! */
#define GALIL1800_WRITE_DELAY_MS_DEFAULT 20

/* Check if value set for write_delay from user
    is correct and eventually correct it
    After convert it from ms to ticks!!
    Return 0 if ok, -EINVAL if reset to default */
static inline int galil1800_write_delay_check_and_convert(unsigned int *wd)
{
	int ret=0;
	if(*wd < 1) {
		*wd = GALIL1800_WRITE_DELAY_MS_DEFAULT;
		printk(KERN_NOTICE PFX "write_delay is less then 1,"
					" set default value (%d)\n", *wd);
		ret=-EINVAL;
	} else if(*wd > 500) {
		*wd = GALIL1800_WRITE_DELAY_MS_DEFAULT;
		printk(KERN_NOTICE PFX "write_delay is more then 500,"
					" set default value (%d)\n", *wd);
		ret=-EINVAL;
	}
	*wd = ((*wd)*HZ)/1000; /* Convert from ms to ticks */
	return ret;
}

/* default milliseconds to wait for check available byte in the read fifo.
   It is initialized in milliseconds and user must set it in ms,
   after, in module init function or ioctl call, it is checked and converted! */
#define GALIL1800_READ_DELAY_MS_DEFAULT 20

/* Check if value set for read_delay from user
    is correct and eventually correct it
    After convert it from ms to ticks!!
    Return 0 if ok, -EINVAL if reset to default */
static inline int galil1800_read_delay_check_and_convert(unsigned int *rd)
{
	int ret=0;
	if(*rd < 1) {
		*rd = GALIL1800_READ_DELAY_MS_DEFAULT;
		printk(KERN_NOTICE PFX "read_delay is less then 1,"
				" so set to default value (%d)\n", *rd);
		ret=-EINVAL;
	} else if(*rd > 500) {
		*rd = GALIL1800_READ_DELAY_MS_DEFAULT;
		printk(KERN_NOTICE PFX "read_delay is more then 500,"
				" so set to default value (%d)\n", *rd);
		ret=-EINVAL;
	}
	*rd = ((*rd)*HZ)/1000; /* Convert from ms to ticks */
	return ret;
}


/* max byte that you can write,
   if GALIL1800_FIFO_HALF_FLAG is not set */
#define GALIL1800_HALF_FULL_WRITE_BURST 255

#define GALIL1800_READ_REG(A)         ((A)->io)
#define GALIL1800_WRITE_REG(A)        GALIL1800_READ_REG(A)
#define GALIL1800_CONTROL_REG(A)      ((A)->ctrl_reg)
#define GALIL1800_IRQ_REG(A)          ((A)->irq_reg)
#define GALIL1800_RESET_REG(A)        GALIL1800_IRQ_REG(A)
#define GALIL1800_2CHAN_REG(A)        ((A)->chan2_reg)

/* FIFO Control register values (size 8 bit) */
#define GALIL1800_FIFO_FULL_FLAG      (0x01)
#define GALIL1800_FIFO_HALF_FLAG      (0x02)
#define GALIL1800_FIFO_EMPTY_FLAG     (0x04)
#define GALIL1800_FIFO2_BUSY_FLAG     (0x08)
#define GALIL1800_FIFO2_FREEZE_FLAG   (0x10)
#define GALIL1800_IRQ_STATUS_FLAG     (0x20)
#define GALIL1800_IRQ_ENABLE_FLAG     (0x40)
#define GALIL1800_FIFO2_EMPTY_FLAG    (0x80)

/* Reset register values (size 8 bit)
   !! Don't use "OR" whit this flag, otherwise the reset don't work !! */
#define __GALIL1800_RESET_FIFO_R    0x02
#define __GALIL1800_RESET_FIFO_W    0x04
#define __GALIL1800_RESET_HARDWARE  0x80

/* Reset timeout in msec */
#define GALIL1800_RESET_TIMEOUT 100

/* Reset the controller: use macro to select what you reset */
static inline void __galil1800_reset(struct galil1800_state *s, char reset)
{
	unsigned long flags;
	spin_lock_irqsave(&s->hwlock, flags);
	outb(reset, GALIL1800_RESET_REG(s));
	spin_unlock_irqrestore(&s->hwlock, flags);
	schedule_timeout_uninterruptible(GALIL1800_RESET_TIMEOUT);
}

#define galil1800_reset_fifo_r(_s)     \
		__galil1800_reset(_s, __GALIL1800_RESET_FIFO_R)

#define galil1800_reset_fifo_w(_s)     \
		__galil1800_reset(_s, __GALIL1800_RESET_FIFO_W)

#define galil1800_reset_fifo_rw(_s)          \
	do {                                 \
		galil1800_reset_fifo_r(_s);  \
		galil1800_reset_fifo_w(_s);  \
	}while(0)

#define galil1800_reset_hardware(_s)      \
	__galil1800_reset(_s, __GALIL1800_RESET_HARDWARE)


static inline void galil1800_enable_IRQ(struct galil1800_state *s)
{
	unsigned long flags;
	spin_lock_irqsave(&s->hwlock, flags);
	inb(GALIL1800_IRQ_REG(s)); /* useless ?! */
	outb(inb(GALIL1800_CONTROL_REG(s)) | GALIL1800_IRQ_ENABLE_FLAG,
						GALIL1800_CONTROL_REG(s));
	inb(GALIL1800_IRQ_REG(s)); /* useless ?! */
	spin_unlock_irqrestore(&s->hwlock, flags);
}

static inline void galil1800_disable_IRQ(struct galil1800_state *s)
{
	unsigned long flags;
	spin_lock_irqsave(&s->hwlock, flags);
	outb(inb(GALIL1800_CONTROL_REG(s)) & ~GALIL1800_IRQ_ENABLE_FLAG,
						GALIL1800_CONTROL_REG(s));
	spin_unlock_irqrestore(&s->hwlock, flags);
}

/* Check for a byte in the Read FIFO
   Return 1 if a byte is available in the FIFO, 0 otherwise. */
static inline int __galil1800_may_read(struct galil1800_state *s)
{
	return !(inb(GALIL1800_CONTROL_REG(s)) & GALIL1800_FIFO_EMPTY_FLAG);
}

/* Wait for a byte in the Read FIFO, or until a timeout occurs.
   Return 1 if a byte is available in the FIFO, 0 otherwise.
   Interrupts MUST be enabled, or the kernel will freeze */
static inline int galil1800_may_read(struct galil1800_state *s)
{
	unsigned long timeout = jiffies + s->read_timeout;
	do {
		if (__galil1800_may_read(s))
			return 1;
	} while (time_before(jiffies,timeout));
	return 0;
}

/* Read from the read fifo */
extern ssize_t galil1800_recv(struct galil1800_state *, char *, ssize_t);

/* Check for a free slot in the Write FIFO, without delay.
   Return:
            2 if FIFO is less than half full (at least 255 free slots),
            1 if a slot is available in the FIFO,
            0 otherwise. */
static inline int __galil1800_may_write(struct galil1800_state *s)
{
	char t = inb(GALIL1800_CONTROL_REG(s));
	if(!(t & GALIL1800_FIFO_HALF_FLAG))
		return 2;
	if(!(t & GALIL1800_FIFO_FULL_FLAG))
		return 1;
	return 0;
}


/* Wait for a free slot in the Write FIFO, or until a timeout occurs.
   Return:
            2 if FIFO is less than half full (at least 255 free slots),
            1 if a slot is available in the FIFO,
            0 otherwise.
   Local interrupts MUST be enabled */
static inline int galil1800_may_write(struct galil1800_state *s)
{
	unsigned long timeout = jiffies + s->write_timeout;
	char t;
	do {
		t = inb(GALIL1800_CONTROL_REG(s));
		if (!(t & GALIL1800_FIFO_HALF_FLAG))
			return 2;
		if (!(t & GALIL1800_FIFO_FULL_FLAG))
			return 1;
	} while (time_before(jiffies,timeout));
	return 0;
}

/* Write data in the write fifo */
extern ssize_t galil1800_send(struct galil1800_state *, const char *, size_t);


/* Secondary Communication Channel:

    - on all DMC-1800, SCC can be used in polling FIFO mode: data goes in
      the secondary FIFO of the controller, and then transferred to the UM
      address space by the driver. Only in Rev. E and greater DPRAM is available,
      but, at this moment, it is not implemented
    - SCC is not active by default, it must be enabled by using the DR command
      with a negative argument -n (2^n samples between updates)
    - DR0 command disables SCC
    - SCC provide information at a fixed rate about position, position error,
      auxiliary position, velocity, torque, axis status, input line status,
      output line status, segment count for coordinate moves, and controller
      status
*/

/* Check for a byte in the secondary FIFO.
   Return 1 if a byte is available in the 2nd FIFO, 0 otherwise */
static inline int __galil1800_may_read_FIFO2(struct galil1800_state *s)
{
	return !(inb(GALIL1800_CONTROL_REG(s)) & GALIL1800_FIFO2_EMPTY_FLAG);
}

/* Wait for a byte in the secondary FIFO, or until a timeout occurs.
   Return 1 if a byte is available in the 2nd FIFO, 0 otherwise.
   Interrupts MUST be enabled, or the kernel will freeze */
static inline int galil1800_may_read_FIFO2(struct galil1800_state *s)
{
	unsigned long timeout = jiffies + s->fifo2_timeout;
	do {
		if (__galil1800_may_read_FIFO2(s))
			return 1;
	} while (time_before(jiffies,timeout));
	return 0;
}

/* Freeze the secondary communication channel */
static inline void galil1800_freeze_FIFO2(struct galil1800_state *s)
{
	unsigned long flags;
	spin_lock_irqsave(&s->hwlock, flags);
	outb(inb(GALIL1800_CONTROL_REG(s)) | GALIL1800_FIFO2_FREEZE_FLAG,
						GALIL1800_CONTROL_REG(s));
	spin_unlock_irqrestore(&s->hwlock, flags);
}

/* Unfreeze the secondary communication channel */
static inline void galil1800_unfreeze_FIFO2(struct galil1800_state *s)
{
	unsigned long flags;
	spin_lock_irqsave(&s->hwlock, flags);
	outb(inb(GALIL1800_CONTROL_REG(s)) & ~GALIL1800_FIFO2_FREEZE_FLAG,
						GALIL1800_CONTROL_REG(s));
	spin_unlock_irqrestore(&s->hwlock, flags);
}

/* Wait until the controller ends transferring a data in the 2nd FIFO */
static inline void galil1800_wait_transfer_FIFO2(struct galil1800_state *s)
{
	while(inb(GALIL1800_CONTROL_REG(s)) & GALIL1800_FIFO2_BUSY_FLAG);
}

/* Read from the secondary communication channel (2nd FIFO) */
extern ssize_t galil1800_read_FIFO2(struct galil1800_state *, char *, size_t);

#endif
